searchNodes=[{"doc":"","ref":"ApiV1.Chores.Example.html","title":"ApiV1.Chores.Example","type":"module"},{"doc":"Callback implementation for ChoreRunner.Chore.inputs/0 .","ref":"ApiV1.Chores.Example.html#inputs/0","title":"ApiV1.Chores.Example.inputs/0","type":"function"},{"doc":"Callback implementation for ChoreRunner.Chore.restriction/0 .","ref":"ApiV1.Chores.Example.html#restriction/0","title":"ApiV1.Chores.Example.restriction/0","type":"function"},{"doc":"Callback implementation for ChoreRunner.Chore.result_handler/1 .","ref":"ApiV1.Chores.Example.html#result_handler/1","title":"ApiV1.Chores.Example.result_handler/1","type":"function"},{"doc":"Callback implementation for ChoreRunner.Chore.run/1 .","ref":"ApiV1.Chores.Example.html#run/1","title":"ApiV1.Chores.Example.run/1","type":"function"},{"doc":"","ref":"ApiV1.Chores.Example.html#validate_input/1","title":"ApiV1.Chores.Example.validate_input/1","type":"function"},{"doc":"A framework and library for productively writing and running code &quot;Chores&quot;. A &quot;Chore&quot; can really be anything, but most commonly it is just some infrequently, manually run code which achieve a business or development goal. For example: updating a config value in a database that does not yet have a UI (perhaps due to time constraints) is a great use for a chore. A chore could be created that accepts the desired value and runs the update query. Usually, the alternative to this would be a direct prod-shell or prod-db connection, which is inherently insecure and dangerous. Many fast-moving startups or companies are ok with this access for developers, and that's fine. But many companies have regulations that they must follow, or do not want to take the risk of a developer mistake while working in these environments. In these cases, ChoreRunner allows the rapid creation, testing, and reviewing of code chores, along with a bundled UI for running them that accepts a variety of input types, with the goal of finding a &quot;sweet spot&quot; of safety and speed when solving such problems. Getting Started Add ChoreRunner to your supervision tree, after your app's PubSub : children = [ { Phoenix.PubSub , [ name : MyApp.PubSub ] } , { ChoreRunner , [ pubsub : MyApp.PubSub ] } , ] Writing a chore defmodule MyApp.MyChore do use ChoreRunner.Chore input :my_file , :file def run ( %{ my_file : path } } ) do path |&gt; File . read! ( ) |&gt; parse_file ( ) |&gt; do_stuff ( ) end end Example of running this Chore: iex&gt; ChoreRunner . run_chore ( MyApp.MyChore , %{ my_file : file } , :infinity ) { :ok , % Chore { } }","ref":"ChoreRunner.html","title":"ChoreRunner","type":"module"},{"doc":"Returns the pubsub topic used for a specific chore, or all chores if given the atom :all","ref":"ChoreRunner.html#chore_pubsub_topic/1","title":"ChoreRunner.chore_pubsub_topic/1","type":"function"},{"doc":"List the currently running chores on all nodes.","ref":"ChoreRunner.html#list_running_chores/0","title":"ChoreRunner.list_running_chores/0","type":"function"},{"doc":"Runs the given chore module as a chore. Accepts an input map with either string or atom keys as well as a keyword list of options. Returns a %ChoreRunner.Chore{} struct. Input map keys must match one of the inputs defined in the provided chore module. If not, the input under the unmatched key is discarded. Matched input will have default validations run on them, as well custom validations declared in the chore module. If any inputs fail validation, the chore will not run, and instead an error tuple will be returned. If all validations pass, the chore will then be run. Opts extra_data: Map of arbitrary data to be forwarded to telemetry events and result handlers. Useful for storing chore session information, such as identifying who or what ran the chore. result_handler: Single arity anonymous function or MFA of a single arity function that is called once the chore is finished. The function will take the %Chore{}.","ref":"ChoreRunner.html#run_chore/3","title":"ChoreRunner.run_chore/3","type":"function"},{"doc":"Stops the provided chore by terminating both the chore task and the reporter. Returns :ok if successful, and :error if not successful","ref":"ChoreRunner.html#stop_chore/1","title":"ChoreRunner.stop_chore/1","type":"function"},{"doc":"Behaviour and DSL for chores.","ref":"ChoreRunner.Chore.html","title":"ChoreRunner.Chore","type":"behaviour"},{"doc":"An optional callback function for defining a chore's inputs. Expects a list of input function calls. The input functions provided are string , int , float , file , and bool . All input functions follow the same syntax. For example: def inputs do [ string ( :name ) , int ( :name2 , [ some : :option ] ) ] end The supported options are :description — a string description of the input, for UI use :validators — a list of anonymous or captured validator functions. Valiator functions should accept a single argument as a parameter, but can return a variety of things, including: an {:ok, value} , or {:error, reason} tuple an :ok or :error atom a true or false any erlang value, or nil The positive values ( :ok , true , non-falsey values) pass validation. The negative values ( :error , false , nil ) fail validation If a value is passed back as part of an {:ok, value} tuple, or by itself, that value is treated as the new value of the given input. This way, validators can also transform input if needed. If this callback is not defined, the default return is [] , or no inputs.","ref":"ChoreRunner.Chore.html#c:inputs/0","title":"ChoreRunner.Chore.inputs/0","type":"callback"},{"doc":"An optional callback function for defining a chore restriction. The restriction can be either :none, :self, or :global :none is no restrictions :self prevents more than one of the same chore from running simultaneously across all connected nodes :global prevents more than one of all chores with the restriction :global from running simultaneously across all connected nodes. This restriction does not affect non- :global chores. If this callback is not defined, the default return is :self","ref":"ChoreRunner.Chore.html#c:restriction/0","title":"ChoreRunner.Chore.restriction/0","type":"callback"},{"doc":"Optional callback to be called once the chore has been completed.","ref":"ChoreRunner.Chore.html#c:result_handler/1","title":"ChoreRunner.Chore.result_handler/1","type":"callback"},{"doc":"A non-optional callback used to contain the main Chore logic. Accepts a map of input, always atom keyed. (When calling ChoreRunner.run_chore/2, a string keyed map will be intelligently converted to an atom-keyed map automatically) Only keys defined in the inputs/0 callback will be present in the input map, but defined inputs are not garaunteed to be present. The chore callback has access to several Reporter functions, used for live chore metrics and loggin. These functions are: log(message) — Logs a string message with timestamp set_counter(name, value) — Sets a named counter, expects an atom for a name and a number for a value inc_counter(name, inc_value) — Increments a named counter. If the counter does not exist, it will default to 0, and then be incremented. Used negative values for decrements. report_failed(reason_message) — Fails a chore, marking it as failed. The return value of the run/1 callback will be stored in the chore struct and forwarded to the final chore handling function.","ref":"ChoreRunner.Chore.html#c:run/1","title":"ChoreRunner.Chore.run/1","type":"callback"},{"doc":"","ref":"ChoreRunner.Chore.html#validate_input/2","title":"ChoreRunner.Chore.validate_input/2","type":"function"},{"doc":"","ref":"ChoreRunner.Chore.html#t:t/0","title":"ChoreRunner.Chore.t/0","type":"type"},{"doc":"","ref":"ChoreRunner.Chore.html#t:unix_timestamp/0","title":"ChoreRunner.Chore.unix_timestamp/0","type":"type"},{"doc":"","ref":"ChoreRunner.ChoreCase.html","title":"ChoreRunner.ChoreCase","type":"module"},{"doc":"","ref":"ChoreRunner.ChoreCase.TestReporter.html","title":"ChoreRunner.ChoreCase.TestReporter","type":"module"},{"doc":"","ref":"ChoreRunner.ChoreCase.TestReporter.html#assert_event/2","title":"ChoreRunner.ChoreCase.TestReporter.assert_event/2","type":"function"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"ChoreRunner.ChoreCase.TestReporter.html#child_spec/1","title":"ChoreRunner.ChoreCase.TestReporter.child_spec/1","type":"function"},{"doc":"Callback implementation for GenServer.init/1 .","ref":"ChoreRunner.ChoreCase.TestReporter.html#init/1","title":"ChoreRunner.ChoreCase.TestReporter.init/1","type":"function"},{"doc":"","ref":"ChoreRunner.ChoreCase.TestReporter.html#start_link/1","title":"ChoreRunner.ChoreCase.TestReporter.start_link/1","type":"function"},{"doc":"Macros which enable the chore DSL","ref":"ChoreRunner.DSL.html","title":"ChoreRunner.DSL","type":"module"},{"doc":"","ref":"ChoreRunner.DSL.html#parse_result_handler/1","title":"ChoreRunner.DSL.parse_result_handler/1","type":"function"},{"doc":"","ref":"ChoreRunner.DSL.html#using/1","title":"ChoreRunner.DSL.using/1","type":"function"},{"doc":"","ref":"ChoreRunner.Input.html","title":"ChoreRunner.Input","type":"module"},{"doc":"","ref":"ChoreRunner.Input.html#bool/2","title":"ChoreRunner.Input.bool/2","type":"function"},{"doc":"","ref":"ChoreRunner.Input.html#file/2","title":"ChoreRunner.Input.file/2","type":"function"},{"doc":"","ref":"ChoreRunner.Input.html#float/2","title":"ChoreRunner.Input.float/2","type":"function"},{"doc":"","ref":"ChoreRunner.Input.html#int/2","title":"ChoreRunner.Input.int/2","type":"function"},{"doc":"","ref":"ChoreRunner.Input.html#string/2","title":"ChoreRunner.Input.string/2","type":"function"},{"doc":"","ref":"ChoreRunner.Input.html#types/0","title":"ChoreRunner.Input.types/0","type":"function"},{"doc":"","ref":"ChoreRunner.Input.html#valid_type/1","title":"ChoreRunner.Input.valid_type/1","type":"macro"},{"doc":"","ref":"ChoreRunner.Input.html#validate_field/2","title":"ChoreRunner.Input.validate_field/2","type":"function"},{"doc":"","ref":"ChoreRunner.Input.html#t:input_options/0","title":"ChoreRunner.Input.input_options/0","type":"type"},{"doc":"","ref":"ChoreRunner.Input.html#t:input_type/0","title":"ChoreRunner.Input.input_type/0","type":"type"},{"doc":"","ref":"ChoreRunner.Input.html#t:reason/0","title":"ChoreRunner.Input.reason/0","type":"type"},{"doc":"","ref":"ChoreRunner.Input.html#t:t/0","title":"ChoreRunner.Input.t/0","type":"type"},{"doc":"","ref":"ChoreRunner.Input.html#t:validator_function/0","title":"ChoreRunner.Input.validator_function/0","type":"type"},{"doc":"","ref":"ChoreRunner.Reporter.html","title":"ChoreRunner.Reporter","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"ChoreRunner.Reporter.html#child_spec/1","title":"ChoreRunner.Reporter.child_spec/1","type":"function"},{"doc":"","ref":"ChoreRunner.Reporter.html#get_chore_state/1","title":"ChoreRunner.Reporter.get_chore_state/1","type":"function"},{"doc":"","ref":"ChoreRunner.Reporter.html#inc_counter/2","title":"ChoreRunner.Reporter.inc_counter/2","type":"function"},{"doc":"Callback implementation for GenServer.init/1 .","ref":"ChoreRunner.Reporter.html#init/1","title":"ChoreRunner.Reporter.init/1","type":"function"},{"doc":"","ref":"ChoreRunner.Reporter.html#log/1","title":"ChoreRunner.Reporter.log/1","type":"function"},{"doc":"","ref":"ChoreRunner.Reporter.html#report_failed/1","title":"ChoreRunner.Reporter.report_failed/1","type":"function"},{"doc":"","ref":"ChoreRunner.Reporter.html#set_counter/2","title":"ChoreRunner.Reporter.set_counter/2","type":"function"},{"doc":"","ref":"ChoreRunner.Reporter.html#start_link/2","title":"ChoreRunner.Reporter.start_link/2","type":"function"},{"doc":"","ref":"ChoreRunner.ReporterSupervisor.html","title":"ChoreRunner.ReporterSupervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"ChoreRunner.ReporterSupervisor.html#child_spec/1","title":"ChoreRunner.ReporterSupervisor.child_spec/1","type":"function"},{"doc":"","ref":"ChoreRunner.ReporterSupervisor.html#start_link/1","title":"ChoreRunner.ReporterSupervisor.start_link/1","type":"function"},{"doc":"A LiveView used in conjunction with ChoreRunner's regular functionality. Features logging and data value display of all running chores locally and on all connected nodes. Allows the running of chores (with autogenerated forms based on chore inputs), as well as stopping running chores. Usage Make sure that Chore.Supervisor is added in your application children = [ { Phoenix.PubSub , name : MyApp.PubSub } , { ChoreRunner , pubsub : MyApp.PubSub } ] A pubsub MUST BE RUNNING and configured for both the Chore supervisor and the Chore LiveView for the Chore UI to function. Make the Chore LiveView accessible in your Phoenix web app by modifying the router. Router @chore_session %{ &quot;otp_app&quot; =&gt; :my_app , &quot;chore_root&quot; =&gt; MyApp.Chores , &quot;pubsub&quot; =&gt; MyApp.PubSub } scope &quot;/&quot; do pipe_through :browser live_session :chores , session : @chore_session do live &quot;/&quot; , ChoreRunnerUI.ChoreLive , :index end end The &quot;chore_root&quot; key in the session should be the module root that all of your chore modules use. For example: if your root is MyApp.Chores your chore modules should be named like MyApp.Chores.MyChore Now you can visit the speficied url and start running chores!","ref":"ChoreRunnerUI.ChoreLive.html","title":"ChoreRunnerUI.ChoreLive","type":"module"},{"doc":"","ref":"ChoreRunnerUI.ChoreView.html","title":"ChoreRunnerUI.ChoreView","type":"module"},{"doc":"The resource name, as an atom, for this view","ref":"ChoreRunnerUI.ChoreView.html#__resource__/0","title":"ChoreRunnerUI.ChoreView.__resource__/0","type":"function"},{"doc":"Renders the given template locally.","ref":"ChoreRunnerUI.ChoreView.html#render/2","title":"ChoreRunnerUI.ChoreView.render/2","type":"function"},{"doc":"Callback invoked when no template is found. By default it raises but can be customized to render a particular template.","ref":"ChoreRunnerUI.ChoreView.html#template_not_found/2","title":"ChoreRunnerUI.ChoreView.template_not_found/2","type":"function"},{"doc":"","ref":"ChoreRunnerUI.Components.ChoreItemComponent.html","title":"ChoreRunnerUI.Components.ChoreItemComponent","type":"module"},{"doc":"Callback implementation for Phoenix.LiveComponent.render/1 .","ref":"ChoreRunnerUI.Components.ChoreItemComponent.html#render/1","title":"ChoreRunnerUI.Components.ChoreItemComponent.render/1","type":"function"},{"doc":"","ref":"ChoreRunnerUI.Components.ChoreModalComponent.html","title":"ChoreRunnerUI.Components.ChoreModalComponent","type":"module"},{"doc":"Callback implementation for Phoenix.LiveComponent.render/1 .","ref":"ChoreRunnerUI.Components.ChoreModalComponent.html#render/1","title":"ChoreRunnerUI.Components.ChoreModalComponent.render/1","type":"function"}]